@@ -2,33 +2,18 @@
 #
 # Copyright (C) 2012 Thomas Doering, falkb
 #
-
-from genshi.builder import tag
-from genshi.filters.transform import Transformer
-from simplemultiproject.model import *
-from trac.util.text import to_unicode
-from trac.core import *
-from trac.web.api import ITemplateStreamFilter, IRequestFilter
-from trac.web.chrome import add_script, add_notice
-from trac.perm import IPermissionPolicy, IPermissionRequestor
-from trac.ticket import model
 from operator import itemgetter
-from smp_model import get_all_versions_without_project
 
-try:
-    from trac.web.chrome import add_script_data
-except ImportError:
-    # Backported from 0.12
-    def add_script_data(req, data={}, **kwargs):
-        """Add data to be made available in javascript scripts as global variables.
+from genshi.filters.transform import Transformer
+from trac.core import *
+from trac.perm import IPermissionPolicy
+from trac.ticket import model
+from trac.util.html import html as tag
+from trac.web.api import ITemplateStreamFilter, IRequestFilter
+from trac.web.chrome import add_script, add_script_data
 
-        The keys in `data` and the keyword argument names provide the names of the
-        global variables. The values are converted to JSON and assigned to the
-        corresponding variables.
-        """
-        script_data = req.chrome.setdefault('script_data', {})
-        script_data.update(data)
-        script_data.update(kwargs)
+from simplemultiproject.model import *
+from simplemultiproject.smp_model import get_all_versions_without_project, SmpProject
 
 
 class SmpTicketProject(Component):
@@ -37,8 +22,10 @@ class SmpTicketProject(Component):
 
     def __init__(self):
         self.__SmpModel = SmpModel(self.env)
+        self.smp_project = SmpProject(self.env)
 
     # IRequestFilter methods
+
     def pre_process_request(self, req, handler):
         return handler
 
@@ -48,8 +35,8 @@ class SmpTicketProject(Component):
             ticket = data.get('ticket', None)
             if ticket:
                 project_name = self.__SmpModel.get_ticket_project(ticket.id)
-                if project_name and project_name[0]:
-                    self.__SmpModel.check_project_permission(req, project_name[0])
+                if project_name:
+                    self.__SmpModel.check_project_permission(req, project_name)
         else:
             is_newticket = True
 
@@ -63,7 +50,7 @@ class SmpTicketProject(Component):
 
             for comp in all_components:
                 components.append(comp.name)
-                comp_projects = [project[0] for project in
+                comp_projects = [project for project in
                                  sorted(self.__SmpModel.get_projects_component(comp.name), key=itemgetter(0))]
                 if comp_projects and len(comp_projects) > 0:
                     component_projects[comp.name] = comp_projects
@@ -71,7 +58,7 @@ class SmpTicketProject(Component):
             all_projects2 = all_projects
             for project in all_projects2:
                 project_versions[project] = ['']
-                project_versions[project].extend([version[0] for version in
+                project_versions[project].extend([version for version in
                                                   self.__SmpModel.get_versions_of_project(project)])
                 project_versions[project].extend(get_all_versions_without_project(self.env))  # See #12463
                 project_versions[project].sort()
@@ -98,12 +85,12 @@ class SmpTicketProject(Component):
 
     def filter_stream(self, req, method, filename, stream, data):
         if filename == "ticket.html":
-            add_script(req, "simplemultiproject/filter_milestones.js")
+            add_script(req, "simplemultiproject/js/filter_milestones.js")
             # replace "project" selection field for ticket with a filtered selection field
             filter = Transformer('//select[@id="field-project"]')
             ticket_data = data['ticket']
 
-            stream = stream | filter.replace(self._projects_field_ticket_input(req, ticket_data))
+            stream |= filter.replace(self._projects_field_ticket_input(req, ticket_data))
 
         return stream
 
@@ -124,15 +111,14 @@ class SmpTicketProject(Component):
         for project in sorted(allProjects, key=itemgetter(1)):
             milestones = self.__SmpModel.get_milestones_of_project(project[1])
             milestonesForProject[project[1]] = {"": ""}
-            for milestone in sorted(milestones, key=itemgetter(0)):
-                ms = milestone[0]
-                is_completed = self.__SmpModel.is_milestone_completed(ms)
+            for milestone in sorted(milestones):
+                is_completed = self.__SmpModel.is_milestone_completed(milestone)
                 if is_newticket:
                     hide_milestone = is_completed
                 else:
                     hide_milestone = not (have_ticketadmin or (not is_completed and have_ticketchgprop))  # see #12201
                 if not hide_milestone:
-                    milestonesForProject[project[1]][ms] = ms
+                    milestonesForProject[project[1]][milestone] = milestone
 
         smp_milestonesForProject = {'smp_milestonesForProject': milestonesForProject}
         smp_initialProjectMilestone = {'smp_initialProjectMilestone': initialProjectMilestone}
@@ -141,8 +127,8 @@ class SmpTicketProject(Component):
         add_script_data(req, smp_milestonesForProject)
 
     def _projects_field_ticket_input(self, req, ticket_data):
-        all_projects = [project[1] for project in sorted(self.__SmpModel.get_all_projects(), key=itemgetter(1))]
-        select = tag.select(name="field_project", id="field-project", onchange="smp_onProjectChange(this.value)")
+        all_projects = [project[1] for project in sorted(self.smp_project.get_all_projects(), key=itemgetter(1))]
+        select = tag.select(name="field_project", id="field-project")
 
         cur_project = ticket_data.get_value_or_default('project')
 
@@ -176,7 +162,7 @@ class ProjectTicketsPolicy(Component):
     * {{{john, mary, group1, authenticated}}} to restrict to this set of users
     * {{{!, bob, anonymous, group2}}} to exclude from the project.
     """
-    implements(IPermissionPolicy, IPermissionRequestor)
+    implements(IPermissionPolicy)
 
     def __init__(self):
         self.__SmpModel = SmpModel(self.env)
@@ -192,11 +178,9 @@ class ProjectTicketsPolicy(Component):
 
         if resource and resource.realm == 'ticket' and resource.id is not None:
             project_name = self.__SmpModel.get_ticket_project(resource.id)
-            if project_name and project_name[0]:
-                project_info = self.__SmpModel.get_project_info(project_name[0])
+            if project_name:
+                project_info = self.__SmpModel.get_project_info(project_name)
                 if project_info:
-                    if self.__SmpModel.is_not_in_restricted_users(username, project_info):
+                    if self.__SmpModel.is_not_in_restricted_users(username,
+                                                                  project_info):
                         return False
-
-    def get_permission_actions(self):
-        return
