@@ -3,66 +3,58 @@
 # Copyright (C) 2012 Thomas Doering
 #
 
-# smp
-from simplemultiproject.model import *
+import re
+from datetime import datetime, timedelta
+from operator import itemgetter
 
-# trac
+from genshi import HTML
+from genshi.filters.transform import Transformer
+from genshi.template.markup import MarkupTemplate
 from trac.attachment import AttachmentModule
 from trac.config import ExtensionOption
 from trac.core import *
-from trac.ticket.model import Version, Ticket, TicketSystem
-from trac.ticket.roadmap import apply_ticket_permissions, get_ticket_stats, ITicketGroupStatsProvider, \
-    DefaultTicketGroupStatsProvider
+from trac.ticket.model import Version, TicketSystem
 from trac.ticket.query import QueryModule
-from trac.mimeview import Context
-from trac import __version__ as VERSION
-from trac.util.translation import _, tag_
-from trac.util.datefmt import parse_date, utc, to_utimestamp, \
-    get_datetime_format_hint, format_date, \
-    format_datetime, from_utimestamp
-from api import IRoadmapDataProvider
-
-try:
-    from trac.util.datefmt import user_time
-except ImportError:
-    def user_time(req, func, *args, **kwargs):
-        """port from 1.0-stable"""
-        if 'tzinfo' not in kwargs:
-            kwargs['tzinfo'] = getattr(req, 'tz', None)
-        return func(*args, **kwargs)
-
+from trac.ticket.roadmap import ITicketGroupStatsProvider, \
+    apply_ticket_permissions, get_ticket_stats
+from trac.web.chrome import web_context
+from trac.util.datefmt import get_datetime_format_hint, parse_date, \
+    user_time, utc
+from trac.util.html import html as tag
+from trac.util.translation import _
 from trac.web.api import IRequestHandler, IRequestFilter, ITemplateStreamFilter
-from trac.web.chrome import add_link, add_notice, add_script, add_stylesheet, \
-    add_warning, Chrome, INavigationContributor
+from trac.web.chrome import add_notice, add_script, add_stylesheet, \
+    add_warning, Chrome
 
-# genshi
-from genshi.builder import tag
-from genshi.filters.transform import Transformer
-
-# python
-from datetime import datetime, timedelta
-from pkg_resources import resource_filename
-from operator import itemgetter
-import re
+from simplemultiproject.admin_filter import SmpFilterDefaultVersionPanels
+from simplemultiproject.api import IRoadmapDataProvider
+from simplemultiproject.model import *
 from simplemultiproject.session import get_filter_settings
+from simplemultiproject.smp_model import SmpVersion
 
-def get_tickets_for_any(env, db, any_name, any_value, field='component'):
-    cursor = db.cursor()
+
+def get_tickets_for_any(env, any_name, any_value, field='component'):
     fields = TicketSystem(env).get_ticket_fields()
     if field in [f['name'] for f in fields if not f.get('custom')]:
-        cursor.execute("""SELECT id,status,%s FROM ticket WHERE %s=%%s ORDER BY %s""" %
-                       (field, any_name, field), [any_value])
+        query = """
+            SELECT id,status,%s FROM ticket WHERE %s=%%s ORDER BY %s
+            """ % (field, any_name, field)
+        args = (any_value,)
     else:
-        cursor.execute("""SELECT id,status,value FROM ticket
-                          LEFT OUTER JOIN ticket_custom ON (id=ticket AND name=%%s)
-                          WHERE %s=%%s ORDER BY value""" % any_name, [field, any_value])
+        query = """
+            SELECT id,status,value FROM ticket
+            LEFT OUTER JOIN ticket_custom ON (id=ticket AND name=%%s)
+            WHERE %s=%%s ORDER BY value
+            """ % any_name
+        args = (field, any_value)
     tickets = []
-    for tkt_id, status, fieldval in cursor:
+    for tkt_id, status, fieldval in env.db_query(query, args):
         tickets.append({'id': tkt_id, 'status': status, field: fieldval})
     return tickets
 
 
-def any_stats_data(env, req, stat, any_name, any_value, grouped_by='component', group=None):
+def any_stats_data(env, req, stat, any_name, any_value, grouped_by='component',
+                   group=None):
     has_query = env[QueryModule] is not None
 
     def query_href(extra_args):
@@ -72,26 +64,31 @@ def any_stats_data(env, req, stat, any_name, any_value, grouped_by='component',
         args.update(extra_args)
         return req.href.query(args)
 
-    return {'stats': stat,
-            'stats_href': query_href(stat.qry_args),
-            'interval_hrefs': [query_href(interval['qry_args']) for interval in stat.intervals]
-            }
+    return {
+        'stats': stat,
+        'stats_href': query_href(stat.qry_args),
+        'interval_hrefs': [query_href(interval['qry_args'])
+                           for interval in stat.intervals]
+    }
 
 
 class SmpVersionProject(Component):
     """Create Project dependent versions"""
 
-    implements(IRequestHandler, IRequestFilter, ITemplateStreamFilter, IRoadmapDataProvider)
+    implements(IRequestFilter, IRequestHandler, IRoadmapDataProvider,
+               ITemplateStreamFilter)
 
-    stats_provider = ExtensionOption('roadmap', 'stats_provider',
-                                     ITicketGroupStatsProvider,
-                                     'DefaultTicketGroupStatsProvider',
-                                     """Name of the component implementing `ITicketGroupStatsProvider`,
+    stats_provider = ExtensionOption(
+        'roadmap', 'stats_provider',
+        ITicketGroupStatsProvider, 'DefaultTicketGroupStatsProvider',
+        """Name of the component implementing `ITicketGroupStatsProvider`,
         which is used to collect statistics on groups of tickets for display
-        in the roadmap views.""")
+        in the roadmap views.
+        """)
 
     def __init__(self):
         self.__SmpModel = SmpModel(self.env)
+        self.smp_version = SmpVersion(self.env)
 
     # IRequestHandler methods
     def match_request(self, req):
@@ -104,7 +101,6 @@ class SmpVersionProject(Component):
     def process_request(self, req):
         version_id = req.args.get('id')
 
-        db = self.env.get_read_db()
         action = req.args.get('action', 'view')
         try:
             version = Version(self.env, version_id)
@@ -120,18 +116,18 @@ class SmpVersionProject(Component):
                 else:
                     req.redirect(req.href.roadmap())
             elif action == 'edit':
-                return self._do_save(req, db, version)
+                return self._do_save(req, version)
             elif action == 'delete':
                 self._do_delete(req, version)
         elif action in ('new', 'edit'):
-            return self._render_editor(req, db, version)
+            return self._render_editor(req, version)
         elif action == 'delete':
-            return self._render_confirm(req, db, version)
+            return self._render_confirm(req, version)
 
         if not version.name:
             req.redirect(req.href.roadmap())
 
-        return self._render_view(req, db, version)
+        return self._render_view(req, version)
 
     # IRequestFilter methods
 
@@ -139,12 +135,15 @@ class SmpVersionProject(Component):
         return handler
 
     def post_process_request(self, req, template, data, content_type):
-        if data and 'is_SMP' in data and req.path_info.startswith('/version'):  # #12371
+        if data and 'is_SMP' in data and \
+                req.path_info.startswith('/version'):  # #12371
             version = data['version']
-            if version and (type(version) is Version):
-                project_name = self.__SmpModel.get_project_version(version.name)
+            if version and type(version) is Version:
+                project_name = \
+                    self.__SmpModel.get_project_version(version.name)
                 if project_name and project_name[0]:
-                    self.__SmpModel.check_project_permission(req, project_name[0])
+                    self.__SmpModel.check_project_permission(req,
+                                                             project_name[0])
         return template, data, content_type
 
     # IRoadmapDataProvider
@@ -163,7 +162,7 @@ class SmpVersionProject(Component):
             data['hide'] = hide
 
         if data and (not hide or 'versions' not in hide):
-            versions, version_stats = self._versions_and_stats(req, None)  # filter_projects)
+            versions, version_stats = self._versions_and_stats(req, None)
             data['versions'] = versions
             data['version_stats'] = version_stats
 
@@ -179,8 +178,7 @@ class SmpVersionProject(Component):
     # ITemplateStreamFilter methods
 
     def filter_stream(self, req, method, filename, stream, data):
-
-        if filename == "version_edit.html":
+        if filename == 'version_edit.html':
             filter_ = Transformer('//form[@id="edit"]/div[1]')
             action = req.args.get('action', 'view')
             if action == 'new':
@@ -194,13 +192,12 @@ class SmpVersionProject(Component):
 
     def __edit_project(self, data, req):
         version = data.get('version').name
-        all_projects = self.__SmpModel.get_all_projects_filtered_by_conditions(req)
+        all_projects = \
+            self.__SmpModel.get_all_projects_filtered_by_conditions(req)
         id_project_version = self.__SmpModel.get_id_project_version(version)
 
-        if id_project_version != None:
-            id_project_selected = id_project_version[0]
-        else:
-            id_project_selected = None
+        id_project_selected = id_project_version \
+            if id_project_version is not None else None
 
         return tag.div(
             tag.label(
@@ -208,14 +205,15 @@ class SmpVersionProject(Component):
                 tag.br(),
                 tag.select(
                     tag.option(),
-                    [tag.option(row[1], selected=(id_project_selected == row[0] or None), value=row[0]) for row in
-                     sorted(all_projects, key=itemgetter(1))],
+                    [tag.option(row[1], selected=(id_project_selected == row[0] or None), value=row[0])
+                     for row in sorted(all_projects, key=itemgetter(1))],
                     name="project")
             ),
             class_="field")
 
     def __new_project(self, req):
-        all_projects = self.__SmpModel.get_all_projects_filtered_by_conditions(req)
+        all_projects = \
+            self.__SmpModel.get_all_projects_filtered_by_conditions(req)
 
         return tag.div(
             tag.label(
@@ -223,7 +221,8 @@ class SmpVersionProject(Component):
                 tag.br(),
                 tag.select(
                     tag.option(),
-                    [tag.option(row[1], value=row[0]) for row in sorted(all_projects, key=itemgetter(1))],
+                    [tag.option(row[1], value=row[0])
+                     for row in sorted(all_projects, key=itemgetter(1))],
                     name="project")
             ),
             class_="field")
@@ -233,16 +232,17 @@ class SmpVersionProject(Component):
         version_name = version.name
         version.delete()
 
-        self.__SmpModel.delete_version_project(version_name)
+        self.smp_version.delete(version_name)
 
         add_notice(req, _('The version "%(name)s" has been deleted.',
                           name=version_name))
         req.redirect(req.href.roadmap())
 
-    def _do_save(self, req, db, version):
+    def _do_save(self, req, version):
         version_name = req.args.get('name')
         version_project = req.args.get('project')
-        old_version_project = self.__SmpModel.get_id_project_version(version.name)
+        old_version_project = \
+            self.__SmpModel.get_id_project_version(version.name)
 
         if version.exists:
             req.perm.require('MILESTONE_MODIFY')
@@ -275,7 +275,7 @@ class SmpVersionProject(Component):
         #        (#4130) and should behave like a WikiPage does in
         #        this respect.
         try:
-            new_version = Version(self.env, new_name, db)
+            new_version = Version(self.env, new_name)
             if new_version.name == old_name:
                 pass  # Creation or no name change
             elif new_version.name:
@@ -287,7 +287,7 @@ class SmpVersionProject(Component):
             version.name = new_name
 
         if warnings:
-            return self._render_editor(req, db, version)
+            return self._render_editor(req, version)
 
         # -- actually save changes
 
@@ -298,31 +298,30 @@ class SmpVersionProject(Component):
                 self.__SmpModel.rename_version_project(old_name, version.name)
 
             if not version_project:
-                self.__SmpModel.delete_version_project(version.name)
+                self.smp_version.delete(version.name)
             elif not old_version_project:
-                self.__SmpModel.insert_version_project(version.name, version_project)
+                self.smp_version.add(version.name, version_project)
             else:
-                self.__SmpModel.update_version_project(version.name, version_project)
+                self.smp_version.update_project_id_for_version(version.name, version_project)
         else:
             version.insert()
             if version_project:
-                self.__SmpModel.insert_version_project(version.name, version_project)
+                self.smp_version.add(version.name, version_project)
 
         add_notice(req, _('Your changes have been saved.'))
         req.redirect(req.href.version(version.name))
 
-    def _render_confirm(self, req, db, version):
+    def _render_confirm(self, req, version):
         req.perm.require('MILESTONE_DELETE')
 
         version = [v for v in Version.select(self.env)
-                   if v.name != version.name
-                   and 'MILESTONE_VIEW' in req.perm]
+                     if v.name != version.name and 'MILESTONE_VIEW' in req.perm]
         data = {
             'version': version
         }
         return 'version_delete.html', data, None
 
-    def _render_editor(self, req, db, version):
+    def _render_editor(self, req, version):
         # Suggest a default due time of 18:00 in the user's timezone
         default_time = datetime.now(req.tz).replace(hour=18, minute=0, second=0,
                                                     microsecond=0)
@@ -345,7 +344,7 @@ class SmpVersionProject(Component):
         Chrome(self.env).add_wiki_toolbars(req)
         return 'version_edit.html', data, None
 
-    def _render_view(self, req, db, version):
+    def _render_view(self, req, version):
         version_groups = []
         available_groups = []
         component_group_available = False
@@ -368,16 +367,13 @@ class SmpVersionProject(Component):
             by = available_groups[0]['name']
         by = req.args.get('by', by)
 
-        tickets = get_tickets_for_any(self.env, db, 'version', version.name, by)
+        tickets = get_tickets_for_any(self.env, 'version', version.name, by)
         tickets = apply_ticket_permissions(self.env, req, tickets)
         stat = get_ticket_stats(self.stats_provider, tickets)
 
-        context = Context.from_request(req)
+        context = web_context(req)
 
-        if VERSION <= '0.12':
-            infodivclass = 'info'
-        else:
-            infodivclass = 'info trac-progress'
+        infodivclass = 'info trac-progress'
 
         data = {
             'context': context,
@@ -400,12 +396,10 @@ class SmpVersionProject(Component):
                         if field.get('optional'):
                             groups.insert(0, '')
                     else:
-                        cursor = db.cursor()
-                        cursor.execute("""
-                            SELECT DISTINCT COALESCE(%s,'') FROM ticket
-                            ORDER BY COALESCE(%s,'')
-                            """, [by, by])
-                        groups = [row[0] for row in cursor]
+                        groups = [row[0] for row in self.env.db_query("""
+                                SELECT DISTINCT COALESCE(%s,'') FROM ticket
+                                ORDER BY COALESCE(%s,'')
+                                """, [by, by])]
 
             max_count = 0
             group_stats = []
@@ -449,7 +443,6 @@ class SmpVersionProject(Component):
         req.perm.require('MILESTONE_VIEW')
 
         versions = Version.select(self.env)
-        db = self.env.get_read_db()
 
         filtered_versions = []
         stats = []
@@ -469,8 +462,8 @@ class SmpVersionProject(Component):
                             version.completed = version.time
 
                     filtered_versions.append(version)
-                    tickets = get_tickets_for_any(self.env, db, 'version', version.name,
-                                                  'owner')
+                    tickets = get_tickets_for_any(self.env, 'version',
+                                                  version.name, 'owner')
                     tickets = apply_ticket_permissions(self.env, req, tickets)
                     stat = get_ticket_stats(self.stats_provider, tickets)
                     stats.append(any_stats_data(self.env, req, stat,
@@ -478,48 +471,33 @@ class SmpVersionProject(Component):
 
         return filtered_versions, stats
 
-######################################################################################################################
-#     Everything below this point is (c) Cinc
-######################################################################################################################
-
-# -*- coding: utf-8 -*-
-#
-# Copyright (C) 2015 Cinc
-#
-# License: 3-clause BSD
-#
-
-from smp_model import SmpVersion
-from genshi import HTML
-from trac.web.chrome import Chrome
-from admin_filter import SmpFilterDefaultVersionPanels
-
 
 class SmpVersionModule(Component):
-    """Module to keep version information for projects up to date when SmpFilterDefaultVersionPanel is deactivated."""
+    """Module to keep version information for projects up to date when
+    SmpFilterDefaultVersionPanel is deactivated.
+    """
     implements(IRequestFilter, ITemplateStreamFilter)
 
     def __init__(self):
         self.smp_model = SmpVersion(self.env)
-        self.version_tmpl = Chrome(self.env).load_template("roadmap_versions.html")
+        chrome = Chrome(self.env)
+        self.version_tmpl = chrome.load_template("roadmap_versions.html", None)
         # CSS class for milestones and versions
-        if VERSION <= '0.12':
-            self.infodivclass = 'info'
-        else:
-            self.infodivclass = 'info trac-progress'
+        self.infodivclass = 'info trac-progress'
 
     # IRequestFilter methods
 
     def pre_process_request(self, req, handler):
         if self._is_valid_request(req) and req.method == "POST":
-            # Try to delete only if version page filter is disabled. Deleting is usually done there.
+            # Try to delete only if version page filter is disabled.
+            # Deleting is usually done there.
             if not self.env.enabled[SmpFilterDefaultVersionPanels]:
                 if 'remove' in req.args:
                     # 'Remove' button on main version panel
-                    self.smp_model.delete(req.args.get('sel'))
+                    self.smp_model.delete(req.args.getlist('sel'))
                 elif 'save' in req.args:
                     # 'Save' button on 'Manage version' panel
-                    p_ids = req.args.get('sel')
+                    p_ids = req.args.getlist('sel')
                     self.smp_model.delete(req.args.get('path_info'))
                     self.smp_model.add_after_delete(req.args.get('name'), p_ids)
         return handler
@@ -527,7 +505,8 @@ class SmpVersionModule(Component):
     @staticmethod
     def _is_valid_request(req):
         """Check request for correct path and valid form token"""
-        if req.path_info.startswith('/admin/ticket/versions') and req.args.get('__FORM_TOKEN') == req.form_token:
+        if req.path_info.startswith('/admin/ticket/versions') and \
+                req.args.get('__FORM_TOKEN') == req.form_token:
             return True
         return False
 
@@ -540,25 +519,41 @@ class SmpVersionModule(Component):
 
         if filename == 'roadmap.html':
             # Add button to create new versions
-            filter_ = Transformer('//div[@class="buttons"][2]')
-            stream = stream | filter_.append(HTML(u'<form action="%s" method="get"><div>'
-                                                  '<input type="hidden" value="new" name="action"/>'
-                                                  '<input value="Add new Version" type="submit">'
-                                                  '</div></form>' % req.href.version()))
+            stream |= self._add_version_button(data)
             # Change label to include versions
             filter_ = Transformer('//label[@for="showcompleted"]')
-            stream = stream | filter_.replace(HTML(u'<label for="showcompleted">Show completed milestones and '
-                                                   'versions</label>'))
+            stream |= filter_.replace(HTML(show_completed_label))
             # Add additional checkboxes to preferences
             data['smp_render'] = 'prefs'
             filter_ = Transformer('//form[@id="prefs"]')
-            stream = stream | filter_.prepend(self.version_tmpl.generate(**data))
+            stream |= filter_.prepend(self.version_tmpl.generate(**data))
 
             # Add versions to page
-            if 'smp_groupproject' not in data:  # Roadmap group plugin is grouping by project
+            # Roadmap group plugin is grouping by project
+            if 'smp_groupproject' not in data:
                 data['infodivclass'] = self.infodivclass
-                data['smp_render'] = 'versions'  # Specify part of template to be rendered
+                # Specify part of template to be rendered
+                data['smp_render'] = 'versions'
                 filter_ = Transformer('//div[@class="milestones"]')
-                stream = stream | filter_.after(self.version_tmpl.generate(**data))
+                stream |= filter_.after(self.version_tmpl.generate(**data))
 
         return stream
+
+    def _add_version_button(self, data):
+        filter_ = Transformer('//div[@class="buttons"][2]')
+        template = MarkupTemplate(add_version_button).generate(**data)
+        return filter_.append(template)
+
+
+show_completed_label = u"""\
+<label for="showcompleted">Show completed milestones and versions</label>
+"""
+
+add_version_button = u"""\
+<form id="add-version" method="get" action="${href.version()}">
+  <div>
+    <input type="hidden" name="action" value="new" />
+    <input type="submit" value="Add new version" />
+  </div>
+</form>
+"""
